////////////////////////////////////////////////////////////////////////////////////////
// Module name: Client.cpp
// Project: NixieKlok-MQTT
// Module: MQTT client, based on PubSubClient library for Arduino.
// by Jan Breemer  jan@breem.nl
//
// Functions in this module:
//
// InitTopics ()
// TopicReceived ()
// UpdateClient ()
// InitClient ()
////////////////////////////////////////////////////////////////////////////////////////

#include "Globals.h"
#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h>
#include <string.h>

#define CLIENTID "NixieKlok"

EthernetClient ethClient;
byte mac[] = {0xDE, 0xED, 0xBA, 0xFE, 0xFE, 0xED };
IPAddress ClientIP (192, 168, 2,  91);
IPAddress BrokerIP (192, 168, 2, 180);
PubSubClient Client(ethClient);

byte HeartBeat;
bool DoPublishClimate, DoPublishHeartBeatAck, DoSynchronize;
bool ConnectionError, SubscriptionError;

// to publish:
char Topic_Climate[30];
char Topic_HeartBeatAck[30];

// to subscribe to:
char Topic_Time[30];
char Payload[25];
char Part[3];

// ***********************************************************************************
void InitTopics (void)  // Called once from InitClient()
{
  // Topics to Subscribe to:
  sprintf (Topic_Time, "TimeServer/Time");
       
  Serial.println ("Prepare Topics to subscribe to: "); 
  Serial.println (Topic_Time);
  Serial.println ();
  
  // Topics to Publish:
  sprintf (Topic_Climate, "%s", CLIENTID);
  strcat  (Topic_Climate, "/Climate");
    
  Serial.println ("Prepare Topics to publish: ");
  Serial.println (Topic_Climate); 
  Serial.println ();  
}

// ***********************************************************************************
void TopicReceived (char* Topic, char* Payload, unsigned int Length) 
{
  if (false) 
  {
    Serial.print ("Message arrived:");
    Serial.print (" Topic= "); 
    Serial.print (Topic);
    Serial.print (" Length= "); 
    Serial.print (Length);
    Serial.print (" PayLoad= "); 
    Serial.println (Payload);
    Serial.println();
  }
    
  if (strcmp(Topic, Topic_Time) == 0)
  {
    Part[0] = Payload[0]; Part[1] = Payload[1]; Part[2] = char(0);
    Year = atoi(Part); 
    Part[0] = Payload[3]; Part[1] = Payload[4];
    Month = atoi(Part);    
    Part[0] = Payload[6]; Part[1] = Payload[7];
    Day = atoi(Part);    
    Part[0] = Payload[9]; Part[1] = char(0);
    DayOfWeek = atoi(Part);
    Part[0] = Payload[11]; Part[1] = Payload[12];
    Hour = atoi(Part);    
    Part[0] = Payload[14]; Part[1] = Payload[15];
    Minute = atoi(Part);
    Part[0] = Payload[17]; Part[1] = Payload[18];
    Second = atoi(Part);
    if (false)
    { 
      Serial.println();
      Serial.println (Year);
      Serial.println (Month); 
      Serial.println (DayOfWeek);
      Serial.println (Day); 
      Serial.println (Hour);  
      Serial.println (Minute); 
    }
    DoSynchronize = true;
  }

  Payload = "";
}

// ***********************************************************************************
void UpdateClient (void) // Called at 1 kHz rate from loop() in .ino file
{
  DIAGPIN_3_T
  Client.loop();
  if (ConnectionError || SubscriptionError) return;
  
  if (DoPublishClimate) 
  { 
    DoPublishClimate = false;
    sprintf (Payload, "%3d,%2d,%6ld", Temperature, Hygro, Baro);
    Client.publish (Topic_Climate, Payload);  
    if (true)
    { 
      Serial.print ("Publish T= ");
      Serial.print (Topic_Climate);
      Serial.print (" Payload = ");
      Serial.println (Payload);
    }  
  }
}

// *********************************************************************************** 
void InitClient (void) // Called once from setup() in .ino file
{
  bool SubscriptionsOK = true;
  Ethernet.begin (mac, ClientIP);
  Client.setServer (BrokerIP, 1883);
  Client.setCallback (&TopicReceived);
  Serial.print ("Client ID = ");
  Serial.println (CLIENTID);
  Serial.print ("Client IP = ");
  Serial.println (ClientIP);
  Serial.print ("Broker IP = ");
  Serial.println (BrokerIP);
  Serial.println();

  InitTopics();
  Serial.println("Try to connect to the MQTT Broker");
  if (Client.connect(CLIENTID)) 
  {
    bool result;
    // connection succeeded
    Serial.print(CLIENTID);
    Serial.println(" is now connected to the MQTT Broker");
    Serial.print("Subscribe to Topic: "); 
    Serial.print(Topic_Time);  
    result = Client.subscribe(Topic_Time);
    if (result) Serial.println("....OK");
    else Serial.println("....FAILED");
  } 
  else 
  {
    Serial.print("Connection WITH MQTT Broker failed. Client.state = ");
    Serial.println(Client.state());
    // mqttClient.state() will provide more information on why it failed.
  }
  Serial.println();
}
